'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _psext = require('psext');

var _psext2 = _interopRequireDefault(_psext);

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _MongoProcess = require('./MongoProcess');

var _MongoProcess2 = _interopRequireDefault(_MongoProcess);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var getSettingsString = function getSettingsString(settings) {
  if (settings.indexOf('{') > -1) return '"' + settings.replace(/"/, '\\"') + '"';
  return settings;
};

var getMongoProcess = function getMongoProcess(meteorPid) {
  return new Promise(function (resolve, reject) {
    _psext2.default.lookup({
      command: 'mongod',
      psargs: '-l',
      ppid: meteorPid
    }, function (err, results) {
      if (err) {
        reject(err);
        return;
      }
      if (results.length > 1) console.warn('Multiple mongo child processes started!'); // eslint-disable-line no-console
      resolve(new _MongoProcess2.default(results[0]));
    });
  });
};

var hasStartedMongoDBText = function hasStartedMongoDBText(buffer) {
  return buffer.lastIndexOf('Started MongoDB') !== -1;
};
var hasErrorText = function hasErrorText(buffer) {
  return buffer.lastIndexOf('Waiting for file change.') !== -1;
};
var hasReadyText = function hasReadyText(buffer) {
  return buffer.lastIndexOf('=> App running at:') !== -1;
};

var MeteorApp = function (_EventEmitter) {
  _inherits(MeteorApp, _EventEmitter);

  function MeteorApp(options) {
    _classCallCheck(this, MeteorApp);

    var _this = _possibleConstructorReturn(this, (MeteorApp.__proto__ || Object.getPrototypeOf(MeteorApp)).call(this));

    _this.args = options.args || [];
    _this.appDir = options.appDir;
    _this.port = options.port;
    _this.settings = options.settings;
    _this.meteorPath = options.meteorPath || _path2.default.basename('meteor');

    _this.buffer = {
      stdout: '',
      stderr: ''
    };

    process.on('exit', function () {
      return _this.kill();
    });
    return _this;
  }

  _createClass(MeteorApp, [{
    key: 'getArgs',
    value: function getArgs() {
      var args = [];
      args.push('run');
      args.push('--port', this.port);
      if (this.settings) args.push('--settings=' + getSettingsString(this.settings));
      this.args.forEach(function (arg) {
        return args.push(arg);
      });
      return args;
    }
  }, {
    key: 'launch',
    value: function launch() {
      var _this2 = this;

      if (!this.launchPromise) {
        this.launchPromise = new Promise(function (resolve, reject) {
          var args = _this2.getArgs();
          _this2.childProcess = _child_process2.default.spawn(_this2.meteorPath, args, {
            cwd: _this2.appDir,
            detached: false
          });

          _this2.childProcess.on('exit', function (code, signal) {
            _this2.emit('exit', code, signal);
          });

          _this2.childProcess.stdout.on('data', function (data) {
            _this2.buffer.stdout += data;
            _this2.emit('stdout', data);

            if (hasStartedMongoDBText(data)) {
              getMongoProcess(_this2.childProcess.pid).then(function (p) {
                _this2.mongoProcess = p;
              });
              _this2.emit('mongodb_started');
            }

            if (hasErrorText(data)) {
              _this2.emit('error', data);
              reject(data);
            }

            if (hasReadyText(data)) {
              _this2.emit('started');
              resolve(_this2);
            }
          });

          _this2.childProcess.stderr.on('data', function (data) {
            _this2.buffer.stderr += data;
            _this2.emit('stderr', data);

            if (hasErrorText(data)) {
              _this2.emit('error', _this2.buffer.stderr);
              reject(data);
            }
          });
        });
      }
      return this.launchPromise;
    }
  }, {
    key: 'kill',
    value: function kill() {
      var signal = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'SIGTERM';

      if (this.childProcess) this.childProcess.kill(signal);
      if (this.mongoProcess) this.mongoProcess.kill();
    }
  }]);

  return MeteorApp;
}(_events.EventEmitter);

exports.default = MeteorApp;